/** 
 * Code for the nRF2401 RF device from "nvlsi.no" - NORDIC
 * using a TI msp430 1232 device. Version 0.1
 *
 * Copyright (C) 05.2004  Paul Kaufmann (paulk@freakmail.de)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
#include "../types.h"
#include "nRF2401.h"

void delay100us(volatile byte n)
{
	volatile byte i;
	while(n--)
		for(i=0;i<150;i++);
}

void nRF2401_configure(	nRF2401_descriptor *desc,
			byte payload_length_1,
			byte payload_length_2,
			byte adress_1,
			byte adress_2,
			byte crc_type,
			byte n_channel_receive,
			byte data_rate,
			byte crystal_freq,
			byte tx_pwr,
			byte rf_channel,
			byte rx_tx_mode)
{
	desc->init_pins();
	
	byte config[15],dummy;
	dummy=256-8-(crc_type==NO_CRC ? 0 : (crc_type==CRC_08 ? 8 : 16)); 	//calculate maximal paylod length 
									 	//addr_length always 8 bits!!!
	config[14]=payload_length_2>dummy ? dummy : payload_length_2;
	config[13]=payload_length_1>dummy ? dummy : payload_length_1;
	config[12]=config[11]=config[10]=config[9]=0;
	config[8]=adress_2;
	config[7]=config[6]=config[5]=config[4]=0;
	config[3]=adress_1;
	config[2]=(8 << 2) | crc_type;						// 8 adress bits !!!!!!!!!!!!!!!!!!!!!
	config[1]=n_channel_receive | SHOCKBURST_MODE | data_rate | crystal_freq | tx_pwr;
	config[0]=(rf_channel << 1) | rx_tx_mode;
	UART0_send_str("\n\r");
	desc->CE(0);
	desc->CS(1);
	delay100us(10);
	nRF2401_pump(desc,OUT,FIRST_IFACE,config,15);
	desc->CS(0);
}
	
void nRF2401_send_data(nRF2401_descriptor *desc, byte adress,  byte *data, byte size){
	desc->CE(1);
	delay100us(1);
	nRF2401_pump(desc,OUT,FIRST_IFACE,&adress,1);
	nRF2401_pump(desc,OUT,FIRST_IFACE,data,size);
	desc->CE(0);
	
}

void nRF2401_receive_data(nRF2401_descriptor *desc, byte rx_channel, byte *data, byte size){
	while (!desc->iface[rx_channel].getDR());
	nRF2401_pump(desc,IN,rx_channel,data,size);
}

void nRF2401_pump(register nRF2401_descriptor *desc, byte dir, byte rx_channel, register byte *data, register byte size){
	register byte mask;
	register void (*CLK)(byte)=desc->iface[rx_channel].CLK;
	
	if (dir==IN){
		register byte (*getDR)(void)=desc->iface[rx_channel].getDR;
		register byte (*getDATA)(void)=desc->iface[rx_channel].getDATA;
		desc->iface[rx_channel].DATA_set_dir(IN);
		while(getDR()){
			if (size)
				size--;
			data[size]=0;
			mask=0x80;
			while(mask){
				CLK(1);
				if (getDATA())
					data[size]|=mask;
				CLK(0);
				mask=mask >> 1;
				}
		}
	}else{ //dir==OUT
		void (*DATA)(byte)=desc->iface[rx_channel].DATA;
		mask=0x80;
		desc->iface[rx_channel].DATA_set_dir(OUT);
		
		while(size--){ //MSB first
			while (mask){
				CLK(0);
				DATA(data[size] & mask);
				CLK(1);
				mask=mask >> 1;
				}
			mask=0x80;
		}
		CLK(0);
	}	
	
}
