/** 
 * Code for the nRF2401 RF device from "nvlsi.no" - NORDIC
 * using a TI msp430 1232 device. Version 0.1
 *
 * Copyright (C) 05.2004  Paul Kaufmann (paulk@freakmail.de)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
#include "../types.h"

#define NO_CRC			0x00
#define CRC_08			0x01
#define CRC_16			0x03

#define ONE_RX_CHANNEL		0x00
#define TWO_RX_CHANNEL		0x80

#define FIRST_IFACE		0x00
#define SECOND_IFACE		0x01

#define DIRECT_MODE		0x00
#define SHOCKBURST_MODE		0x40

#define DATA_RATE_250KBPS	0x00
#define DATA_RATE_1MBPS		0x20

#define CRYSTAL_04_MHZ		0x00
#define CRYSTAL_08_MHZ		0x04
#define CRYSTAL_12_MHZ		0x08
#define CRYSTAL_16_MHZ		0x0C
#define CRYSTAL_20_MHZ		0x10

#define POWER_MINUS_20_dBm	0x00
#define POWER_MINUS_10_dBm	0x01
#define POWER_MINUS_05_dBm	0x02
#define POWER_MINUS_00_dBm	0x03

#define TRANSMIT_MODE		0x00
#define RECEIVE_MODE		0x01

#define IN			0x00
#define OUT			0x01

typedef struct{
	void (*CLK)		(byte data);
	byte (*getDR)		(void);
	void (*DATA_set_dir)	(byte dir);
	void (*DATA)		(byte data);
	byte (*getDATA)		(void);
} three_wire_iface;

typedef struct{
	void (*init_pins)	(void);
	void (*CE)		(byte data);
	void (*CS)		(byte data);
	three_wire_iface iface[2];
}nRF2401_descriptor;

void nRF2401_configure(	nRF2401_descriptor *desc,
			byte payload_length_1,
			byte payload_length_2,
			byte adress_1,
			byte adress_2,
			byte crc_type,
			byte n_channel_receive,
			byte data_rate,
			byte crystal_freq,
			byte tx_pwr,
			byte rf_channel,
			byte rx_tx_mode);
		
void nRF2401_send_data(nRF2401_descriptor *desc, byte adress,  byte *data, byte size);
void nRF2401_receive_data(nRF2401_descriptor *desc, byte rx_channel, byte *data, byte size);
void nRF2401_pump(nRF2401_descriptor *desc, byte dir, byte rx_channel, byte *data, byte size);

