/** 
 * Code for the nRF2401 RF device from "nvlsi.no" - NORDIC
 * using a TI msp430 1232 device. Version 0.1
 *
 * Copyright (C) 05.2004  Paul Kaufmann (paulk@freakmail.de)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 

#include "../types.h"
#include "nRF2401.h"
#include "../rs232/rs232.h"
#include <msp430x12x2.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <iomacros.h>
#include <signal.h>
#include <math.h>


nRF2401_descriptor desc0;
nRF2401_descriptor desc1;

void init_pins_0(void);
void init_pins_1(void);



void init_pins_0(void){
	UART0_tx_wait();
	P1SEL &=0xfe;		//I/O for P1.0
	P1DIR &=0xfe;		//P1.0 has now input direction
	P3SEL &=0xf0;		//I/O for P3.0 - P3.3
	P3DIR |=0x0f;		//P3.0 - P3.3 has now output direction
}

void init_pins_1(void){
	P1SEL &=0xfd;		//I/O for P1.1
	P1DIR &=0xfd;		//P1.1 has now input direction
	P2SEL &=0xcf;		//P2.4, P2.5 for I/O
	P2DIR |=0x10;		//P2.4 has now output direction
	P2DIR &=0xdf;		//P2.5 has now input direction
	P3SEL &=0x3f;		//P3.6, P3.7 for I/O
	P3DIR |=0xc0;		//P3.6, P3.7 has now output direction
}

//--------------------------------------------------------------------------
void CE_0(byte data){
	if (data)
		P3OUT |= 0x01;
	else //data==0
		P3OUT &= ~0x01;
}

void CS_0(byte data){
	if (data)
		P3OUT |= 0x02;
	else //data==0
		P3OUT &= ~0x02;
}
	
void CLK_0_0(byte data){ 
	if (data) 
		P3OUT |= 0x04; 
	else 
		P3OUT &= ~0x04; 
}

byte getDR_0_0(void){
	return P1IN & 0x01;	
}

void DATA_set_dir_0_0(byte dir){
	if (dir==IN)
		P3DIR &= ~ 0x08;
	else //dir==OUT
		P3DIR |= 0x08;
}

void DATA_0_0(byte data){
	if (data)
		P3OUT |= 0x08;
	else //data==0
		P3OUT &= ~0x08;	
}

byte getDATA_0_0(void){
	return (P3IN & 0x08)==0x08;
}

//--------------------------------------------------------------------------	
void CE_1(byte data){
	if (data)
		P3OUT |= 0x80;
	else //data==0
		P3OUT &= ~0x80;
}

void CS_1(byte data){
	if (data)
		P3OUT |= 0x40;
	else //data==0
		P3OUT &= ~0x40;
}
	
void CLK_1_0(byte data){ 
	if (data) 
		P2OUT |= 0x10; 
	else 
		P2OUT &= ~0x10; 
}

byte getDR_1_0(void){
	return (P1IN & 0x02)==0x02;	
}

void DATA_set_dir_1_0(byte dir){
	if (dir==IN)
		P2DIR &= ~ 0x20;
	else //dir==OUT
		P2DIR |= 0x20;
}

void DATA_1_0(byte data){
	if (data)
		P2OUT |= 0x20;
	else //data==0
		P2OUT &= ~0x20;	
}

byte getDATA_1_0(void){
	return (P2IN & 0x20)==0x20;
}
//--------------------------------------------------------------------------	

void init(void){
	//set maximal CPU speed
	DCOCTL = DCO0 + DCO1 + DCO2 + MOD4 + MOD3;
	DCOCTL = DCO1 + DCO2;
	BCSCTL1 = RSEL2 + RSEL1 + RSEL0;
	BCSCTL2 = 0x00;
	//stop watchdog
	WDTCTL = WDTPW + WDTHOLD;	//stop watchdog
	//rs232 init
	UART0_init();
}
	
	
void test_pins(){
	uprintf(UART0_send_char,"CE_0, CS_0, CLK_0_0, DATA_0_0, CE_1, CS_1, CLK_1_0 and DATA_1_0 sholud be 1\n\r");
	CE_0(1);
	CS_0(1);
	CLK_0_0(1);
	DATA_set_dir_0_0(OUT);
	DATA_0_0(1);
	CE_1(1);
	CS_1(1);
	CLK_1_0(1);
	DATA_set_dir_1_0(OUT);
	DATA_1_0(1);
	UART0_rec_char();
	uprintf(UART0_send_char,"CE_0, CS_0, CLK_0_0, DATA_0_0, CE_1, CS_1, CLK_1_0 and DATA_1_0 sholud be 0\n\r");
	CE_0(0);
	CS_0(0);
	CLK_0_0(0);
	DATA_set_dir_0_0(OUT);
	DATA_0_0(0);
	CE_1(0);
	CS_1(0);
	CLK_1_0(0);
	DATA_set_dir_1_0(OUT);
	DATA_1_0(0);
	UART0_rec_char();
	
	DATA_set_dir_0_0(IN);
	DATA_set_dir_1_0(IN);
	
	while (!UART0_rec_avail()){
		uprintf(UART0_send_char,"DR_0_0: %i  DATA_0_0: %i  DR_1_0: %i  DATA_1_0: %i\n\r",
			getDR_0_0(),getDATA_0_0(),getDR_1_0(),getDATA_1_0());
		}
	
}
	
int main(){
	
	desc0.init_pins=init_pins_0;
	desc0.CE=(*CE_0);
	desc0.CS=(*CS_0);
	desc0.iface[0].CLK=(*CLK_0_0);
	desc0.iface[0].getDR=(*getDR_0_0);
	desc0.iface[0].DATA_set_dir=DATA_set_dir_0_0;
	desc0.iface[0].DATA=(*DATA_0_0);
	desc0.iface[0].getDATA=(*getDATA_0_0);
	
	desc1.init_pins=(*init_pins_1);
	desc1.CE=(*CE_1);
	desc1.CS=(*CS_1);
	desc1.iface[0].CLK=(*CLK_1_0);
	desc1.iface[0].getDR=(*getDR_1_0);
	desc1.iface[0].DATA_set_dir=(*DATA_set_dir_1_0);
	desc1.iface[0].DATA=(*DATA_1_0);
	desc1.iface[0].getDATA=(*getDATA_1_0);
	
	init();
	//init_pins_0();
	//init_pins_1();
	//test_pins();
	
	
	#define payload 5
	nRF2401_configure(	&desc0,payload*8,0,1,0x10,CRC_16,ONE_RX_CHANNEL,DATA_RATE_250KBPS,
				CRYSTAL_16_MHZ,POWER_MINUS_00_dBm,120,TRANSMIT_MODE);
	nRF2401_configure(	&desc1,payload*8,0,2,0x20,CRC_16,ONE_RX_CHANNEL,DATA_RATE_250KBPS,
				CRYSTAL_16_MHZ,POWER_MINUS_00_dBm,120,RECEIVE_MODE);
				
	byte i;
	byte data[payload];
	volatile word ww;
	
	desc1.CE(1);
	UART0_send_str("\n\r");
	unsigned int counter_send=0,counter_rec=0,counter_fail=0;
	ww=1;
	while(1){
		ww--;
		if (!ww){
			for (i=0;i<payload;i++)	
				data[i]=i+'a';
			nRF2401_send_data(&desc0,2,data,payload);
			counter_send++;
			uprintf(UART0_send_char,"\n\r%.6i %.6i %.6i\n\r",counter_send,counter_rec,counter_fail);
		}
			
		
		if (desc1.iface[0].getDR()){
			nRF2401_receive_data(&desc1, 0, data, payload);
			for (i=0;i<payload && data[i]==i+'a';i++);	
			if (i==payload)
				counter_rec++;
			else
				counter_fail++;
			uprintf(UART0_send_char,"%.6i %.6i %.6i\n\r",counter_send,counter_rec,counter_fail);
			ww=1;
		}
		
	}
}

